% move takes as input:
%  dlist:   list of target positions, relative to zero, in encoder counts
%  tmlist:  list of times to get to each successive position, in sample times
%  trlist:  list of times to ramp up (and down) to coast velocities
%            (should be 1/2 of the motion time tm or less)
%  th:      the time to hold the start and end position
% and returns a trajectory consisting of encoder counts (expressed as floating
% point) as a function of sample count.

function motion = move(dlist,tmlist,trlist,th)

if ((length(dlist)~=length(tmlist)) || ...
  ((length(tmlist)~=length(trlist))))
  disp('Error:  first three arguments must be the same length.');
  return;
end
motion = 0;
motion(1:th) = 0;  
currentlength = th;
len = length(dlist);
if (len>1)
  dlist(2:len) = diff(dlist);
end
for i=1:length(dlist)
  trajpiece = trapezoid(dlist(i),tmlist(i),trlist(i));
  trajpiece = trajpiece+motion(currentlength);
  len = length(trajpiece);
  motion(currentlength+1:currentlength+len) = trajpiece;
  currentlength = currentlength+len;
end
motion(currentlength+1:currentlength+th) = motion(currentlength);


% trapezoid takes as input:
%  d:  distance to move in encoder counts
%  tm: total time for the motion in sample time counts
%  tr: speed ramp-up time (should be tm/2 or less)
% and returns a trajectory consisting of encoder counts (expressed as floating 
% point) as a function of sample count 

function traj = trapezoid(d, tm, tr)

traj = 0;      % clear the trajectory
if (tm<2) 
  tm = 2;      % minimum motion time
end
if (tr<1)
  tr = 1;      % minimum velocity ramp-up time
end
if (2*tr>tm)   % ramp can be no longer than half the trajectory
  tr = tm/2;
end
tc = tm-2*tr;  % time for coasting at constant velocity
acc = d/(tr^2 + tr*tc);        % acceleration during velocity ramp up
coastvel = acc*tr;             % velocity to coast at
coaststartpos = 0.5*acc*tr^2;  % starting position of coast
coastendpos = d-coaststartpos; % ending position of coast
i1 = tr; i2 = tr+tc; i3 = 2*tr+tc; 
for i=0:floor(i1)    % ramp-up samples
  ts = i;
  traj(i+1) = 0.5*acc*ts^2;
end
for i=floor(i1)+1:floor(i2)    % coast samples
  ts = i-tr;
  traj(i+1) = coaststartpos + coastvel*ts;
end
for i=floor(i2)+1:floor(i3)    % ramp-down samples
  ts = i-tr-tc;
  traj(i+1) = coastendpos + coastvel*ts - 0.5*acc*ts^2;
end








